/*
 *  FileSystem.es -- FileSystem class
 *
 *  Copyright (c) All Rights Reserved. See details at the end of the file.
 */

module ejs.io {

    /**
     *  @spec ejs-12
     */
    native class FileSystem {

        use default namespace public

//TODO is this right to give a path?
//TODO who creates the default file system
        /**
         *  Create a new FileSystem object based on the given path.
         *  @param path String or Path of the file system
         */
        native function FileSystem(path: Object)

        /**
         *  Return the free space in the file system.
         *  @return The number of 1M blocks (1024 * 1024 bytes) of free space in the file system.
         */
        #FUTURE
        native function get space(): Number

        /**
         *  Determine if path names have a drive specifications (C:). Only relevant on Windows like file systems.
         *  @return True if the file path has a drive spec
         */
        native function get hasDrives(): Boolean 

        /**
         *  Determine if the file system is ready for use
         *  @return true If the file system is mounted and available
         */
        #FUTURE
        native function get isReady(): Boolean

        /**
         *  Determine if the file system is writable
         *  @return true If the file system will accept write requests
         */
        #FUTURE
        native function get isWritable(): Boolean

        /**
         *  Return the new line characters for this file system
         *  @return the new line delimiting characters Usually "\n" or "\r\n".
         */
        native function get newline(): String 

        /**
         *  Set the new line characters to use for this file system
         *  @param terminator the new line termination characters Usually "\n" or "\r\n"
         */
        native function set newline(terminator: String): Void

        /**
         *  Return the root directory of the file system
         *  @returns a Path which represents the file system
         */
        native function get root(): Path

        /**
         *  Return the path segment separators
         *  @return the path segment separators. The first character is the default separator. Usually "/" or "\\"
         */
        native function get separators(): String 

        /**
         *  Set the path segment separators. The first separator is the default separator.
         *  @param sep the new path segment separator. Usually "/" or "/\\"
         */
        native function set separators(sep: String): Void 

        /**
         *  Return the total size of the file system.
         *  @return The number of 1M blocks (1024 * 1024 bytes) in the file system.
         */
        #FUTURE
        native function get size(): Number
    }
}

/*
 *  @copy   default
 *  
 *  Copyright (c) Embedthis Software LLC, 2003-2009. All Rights Reserved.
 *  Copyright (c) Michael O'Brien, 1993-2009. All Rights Reserved.
 *  
 *  This software is distributed under commercial and open source licenses.
 *  You may use the GPL open source license described below or you may acquire 
 *  a commercial license from Embedthis Software. You agree to be fully bound 
 *  by the terms of either license. Consult the LICENSE.TXT distributed with 
 *  this software for full details.
 *  
 *  This software is open source; you can redistribute it and/or modify it 
 *  under the terms of the GNU General Public License as published by the 
 *  Free Software Foundation; either version 2 of the License, or (at your 
 *  option) any later version. See the GNU General Public License for more 
 *  details at: http://www.embedthis.com/downloads/gplLicense.html
 *  
 *  This program is distributed WITHOUT ANY WARRANTY; without even the 
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  
 *  This GPL license does NOT permit incorporating this software into 
 *  proprietary programs. If you are unable to comply with the GPL, you must
 *  acquire a commercial license to use this software. Commercial licenses 
 *  for this software and support services are available from Embedthis 
 *  Software at http://www.embedthis.com 
 *  
 *  @end
 */
