/*
 *  Path testing
 */

const TestLength = 500
const TestPath: String = "file.dat"

if (App.dir.basename == "test") {
    App.chdir("io")
}

//  Constructor

Path("/a/b/c")
Path("")
Path(Path("/a/b/c"))

//	Test basic file attributes

p = new Path(TestPath)
assert(p.exists)
assert(p.isRegular)
assert(!p.isDir)
assert(!p.isOpen)
assert(p.isRelative)
assert(!p.isAbsolute)
//FUTURE    assert(!p.isSymbolic)
//FUTURE    assert(!p.isLink)
assert(!p.hasDrive)
assert(p.name == "file.dat")
assert(p.size == TestLength)
assert(p.length == TestPath.length)
assert(p.created is Date)
assert(p.accessed is Date )
assert(p.modified is Date )
if (p.hasDriveSpecs) {
	assert(p.name[0].isAlpha)
}

// FUTURE //  attributes
// FUTURE 
// FUTURE p = new Path(TestPath)
// FUTURE att = p.attributes
// FUTURE assert(att is Object && att.length > 0)
// FUTURE assert(att.permissions == 0644)
// FUTURE assert(att.owner is String && att.owner.length > 0)
// FUTURE assert(att.group is String && att.group.length > 0)

//  base, directory, extension, 

p = Path("/a/b/c.dat")
assert(p.base == "c.dat")
assert(p.directory == "/a/b")
assert(p.parent.base == "b")
assert(p.extension == ".dat")

//	absolute, relative

p = Path("file.dat")
assert(p.parent.isAbsolute)
assert(p.relative == "file.dat")
assert(Path("a.b").relative == "a.b")
assert(Path("./a.b").relative == "./a.b")
assert(Path("/tmp/a.b").relative.toString().indexOf("../") >= 0)
assert(Path("/tmp/a.b").relative.isRelative)
assert(Path("/tmp/a.b").absolute.isAbsolute)

//  portable, natural

assert(p.portable == "file.dat")
if (Config.OS == "WIN") {
    assert(Path("c:\\a\\b\\c").portable == "/a/b/c")
    assert(Path("\\a\\b\\c").portable == "/a/b/c")
    p = Path("/a/b/c").natural
    assert(p.isAbsolute)
    assert(p.indexOf('\\') >= 0)
}

//  normalize

assert(Path("////a/b/c").normalize == "/a/b/c")
assert(Path("////a/b/c////").normalize == "/a/b/c")
assert(Path("a/././b/c").normalize == "a/b/c")
assert(Path("/a/b/../c").normalize == "/a/c")
assert(Path("/a/b/../c/.").normalize == "/a/c")

//  startsWith, endsWith

p = Path("/a/b/c.gif")
assert(p.startsWith("/a"))
assert(p.endsWith(".gif"))

//  tosString

p = Path("/a/b/c.gif")
assert(p.toString() == "/a/b/c.gif")

//  join

assert(Path("/tmp").join("my.dat") == "/tmp/my.dat")
assert(Path("/tmp").join("/my.dat") == "/my.dat")
assert(Path("/tmp").join("////my.dat") == "/my.dat")
assert(Path("/tmp").join("./my.dat") == "/tmp/my.dat")
assert(Path("/tmp").join("./././my.dat") == "/tmp/my.dat")
assert(Path("").join("./././my.dat") == "my.dat")
assert(Path("/tmp").join("") == "/tmp")
assert(Path("/tmp").join("") == "/tmp")

//  joinExt

assert(Path("/a/b/c").joinExt(".dat") == "/a/b/c.dat")
assert(Path("/a/b/c.mod").joinExt(".dat") == "/a/b/c.mod")

//  same

assert(Path("./Makefile").same("Makefile"))

assert(Path("../../test/io/Makefile").same("Makefile"))
assert(Path("../../test/io/Makefile").absolute.same("Makefile"))
assert(Path("../../test/io/Makefile").relative.same("Makefile"))

//  copy and remove

name = "temp.dat"
Path(TestPath).copy(name)
p = new Path(name)
assert(p.exists)
assert(p.size == TestLength)
p.remove()
assert(!p.exists)

//  rename

name = "temp.dat"
p = new Path(name)
Path(TestPath).copy(name)
assert(p.exists)
p.rename("another.dat")
assert(Path("another.dat").exists)
Path("another.dat").remove()

//  truncate

//FUTURE name = "temp.dat"
//FUTURE p = new Path(name)
//FUTURE Path(TestPath).copy(name)
//FUTURE assert(p.size == TestLength)
//FUTURE p.truncate(20)
//FUTURE assert(p.size == 20)
//FUTURE p.remove()

//  files

files = Path("..").files(true)
assert(files is Array)
assert(files.length > 5)
assert(files.find(function (e) { return e.same("../core"); }) == "../core")

//	makeDir, removeDir

p = new Path("tempDir")
p.removeDir()
assert(!p.exists)
p.makeDir()
assert(p.exists)
assert(p.isDir)
p.removeDir()
assert(!p.exists)

// makeLink, isLink, isSymbolic - TODO

//  open

file = Path(TestPath).open({ mode: "r" })
buf = new ByteArray()
count = file.read(buf)
assert(buf.available == TestLength)
file.close()

// openTextStream

stream = Path(TestPath).openTextStream({ mode: "r" })
line = stream.readLine()
assert(line = "123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789")
stream.close()

//  openBinaryStream

stream = Path(TestPath).openBinaryStream({ mode: "r" })
assert(stream.readByte() == 49)
assert(stream.readByte() == 50)
assert(stream.readByte() == 51)
stream.close()

//	readBytes

bytes = Path(TestPath).readBytes()
assert(bytes is ByteArray)
assert(bytes.length == TestLength)
assert(bytes[0] == 49)
assert(bytes.readString(1) == '1')

//	readLines

lines = Path(TestPath).readLines()
for each (l in lines) {
	assert(l.search("12345") == 0);
}

// readString

str = Path(TestPath).readString()
assert(str.length == 500)
assert(str.contains("123456789"))

// readXml

xml = Path("file.xml").readXML()
assert(xml is XML)
assert(xml.customer != null)

//	write 

Path("temp.dat").write("Hello World\r\n", new Date, "\r\n")
p = Path("temp.dat")
assert(p.exists)
p.remove()
assert(!p.exists)

if (Config.OS == "WIN") {
    p = Path("/a/b/c").absolute
    p.setRepresentation(Path.NATIVE)
    assert(p.indexOf("\\") >= 0)
    assert(p.hasDrive)
}

//  components

parts = Path("/a/b/c").components
assert(parts.length == 4)
assert(parts[0] == "")
assert(parts[1] == "a")
assert(parts[2] == "b")
assert(parts[3] == "c")
assert(parts.join("/") == "/a/b/c")

//  truncate

name = "temp.dat"
Path(TestPath).copy(name)
p = new Path(name)
assert(p.exists)
assert(p.size == TestLength)
p.truncate(10)
assert(p.size == 10)
p.remove()

//  makeTemp

p = Path(".").makeTemp()
assert(p.exists)
p.remove()
