/*
 *	Length in bytes of file.dat
 */
const TestLength = 500
const TestFile: String = "file.dat"

if (App.dir.basename == "test") {
    App.chdir("io")
}

//	readBytes

f = new File(TestFile)
f.open("r")
assert(f.isOpen)
assert(f.canRead)
assert(!f.canWrite)
bytes = f.readBytes(1024)
assert(f.options.mode == "r")
assert(f.options.permissions == 0664)
assert(bytes.length == 1024)
assert(bytes.available == TestLength)

//	position

f.position = 0
again = f.readBytes(1024)
assert(again.length == bytes.length)
assert(again.available == TestLength)

//	write

f = new File("temp.dat", { mode: "w", permissions: 0644 })
f.write("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789\r\n")
f.flush()
assert(f.size == 102)
f.close()
assert(!f.isOpen)

//  path

path = f.path
assert(path.exists)
// assert(path.permissions = 0644)
// path.permissions = 777
// assert(path.permissions = 0777)
f.remove()
assert(!path.exists)

//	index access

f = new File("temp.dat", { mode: "w" })
for (i in 256) {
	f[i] = i
}
f.close()
assert(!f.isOpen)
f.open()
for (i in 256) {
	assert(f[i] == i)
}
f.close()

//	Read via direct indexing 

f = new File("temp.dat")
f.open()
count = 0
for (i in f) {
	assert(f[i] == i)
	count++
}
assert(count == 256)
f.close()
f.remove()
assert(!f.path.exists())

//  truncate

Path(TestFile).copy("temp.dat")
assert(Path("temp.dat").exists)
f = new File("temp.dat")
f.truncate(5)
assert(f.size == 5)
assert(f.path.size == 5)
f.remove()
